/**
 * Copyright 2007 ZeroTurnaround.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

package org.zeroturnaround.javarebel;

/**
 * Provides methods to reload classes and receive notifications when they were reloaded. 
 * 
 * @author Jevgeni Kabanov (ekabanov@zeroturnaround.com)
 */
public interface Reloader {
  /**
   * Returns whether reloading is enabled generally.
   * @return Whether reloading is enabled generally.
   */
  boolean isReloadEnabled();
  
  /**
   * Returns whether this particular class will be reloaded when it is updated.
   * @param klass Any class.
   * @return Whether this particular class will be reloaded when it is updated.
   */
  boolean isReloadableClass(Class klass);
  
  /**
   * <p>Causes a class to be reloaded if (and only if) the 
   * underlying ".class" file has been updated. Returns
   * <b>true</b> if reloaded, <b>false</b> otherwise.</p>
   * 
   * <p>Note, that as JavaRebel is lazy such checks may be necessary
   * if you want to be sure that your metadata is up-to-date. 
   * The {@link ReloadListener} will only fire after JavaRebel 
   * has reloaded the class, which by default occurs on method 
   * calls to the class in question (and some reflection methods).</p>
   * 
   * @param klass The class to check.
   * @return <b>true</b> if reloaded, <b>false</b> otherwise.
   * 
   * @see ReloadListener
   */
  boolean checkAndReload(Class klass);  
  
  /**
   * Register a {@link ReloadListener} to receive notifications on class load.
   * 
   * @param listener Listener to receive notifications  on class load.
   * 
   * @see ReloadListener
   */
  void addLoadListener(ReloadListener listener);
  
  /**
   * Deregister a {@link ReloadListener}.
   * 
   * @param listener Listener to deregister.
   * 
   * @see ReloadListener
   */

  void removeLoadListener(ReloadListener listener);
  
  /**
   * Register a {@link ReloadListener} to receive notifications on class load.
   * 
   * @param listener Listener to receive notifications  on class load.
   * 
   * @see ReloadListener
   */
  void addReloadListener(ReloadListener listener);
  
  /**
   * Deregister a {@link ReloadListener}.
   * 
   * @param listener Listener to deregister.
   * 
   * @see ReloadListener
   */

  void removeReloadListener(ReloadListener listener);
  
  boolean isManagedClass(String classname, ClassResource cr);
  
  //TODO:
  // * Registering bytecode transformers
  // * Replacing bytecode
}
